/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>

#pragma implementation "hem.hxx"
#pragma implementation "hem.cc"

#include "hem.hxx"
#include "hem.cc"


#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>

struct timeval Tbegin;
struct timeval Tend;
struct timeval tt ;

void timeBegin()
{
  gettimeofday(&Tbegin, NULL);
}

void timeEnd()
{
  gettimeofday(&Tend, NULL);
}

double timeDiff()
{
  double res = (Tend.tv_sec - Tbegin.tv_sec) ;
  res += ((float)(Tend.tv_usec)/1000000) - ((float)(Tbegin.tv_usec)/1000000) ;
  return res ;
}

/*
struct OidInd {
  unsigned int lev7 : 4 ;
  unsigned int lev6 : 4 ;
  unsigned int lev5 : 4 ;
  unsigned int lev4 : 4 ;
  unsigned int lev3 : 4 ;
  unsigned int lev2 : 4 ;
  unsigned int lev1 : 4 ;
  unsigned int lev0 : 4 ;
} ;
*/

struct OidInd {
  unsigned int lev6 : 3 ;
  unsigned int lev5 : 3 ;
  unsigned int lev4 : 3 ;
  unsigned int lev3 : 3 ;
  unsigned int lev2 : 3 ;
  unsigned int lev1 : 3 ;
  unsigned int lev0 : 14 ;
} ;

struct OidInfo {
  unsigned int volNo   : 6 ; 	// Volume number 
  unsigned int segNo   : 8 ;	// Segment number
  unsigned int pageNo  : 10 ;	// Page number
  unsigned int objNo   : 8 ;    // object number or number of page
} ; 


class OID {
  public :

  union {
    OidInfo info ;
    int     value ;
  } ;
  
  OID() {}
  OID(const OID& oid) { this->value = oid.value ; }
  OID(int val) { this->value = val ; }
  int operator==(const OID& oid) const { return this->value == oid.value ; }
  unsigned getRootInd() { return ((OidInd*)this)->lev0 ; }
  unsigned getNodeInd(int lev) {
    if (lev == 1) return ((OidInd*)this)->lev1 ;
    else if (lev == 2) return ((OidInd*)this)->lev2 ;
    else if (lev == 3) return ((OidInd*)this)->lev3 ;
    else if (lev == 4) return ((OidInd*)this)->lev4 ;
    else if (lev == 5) return ((OidInd*)this)->lev5 ;
    else if (lev == 6) return ((OidInd*)this)->lev6 ;
    else if (lev == 0) return ((OidInd*)this)->lev0 ;
  }
} ;


class Page {  
  public :

  OID oid ;
  Page(OID& o) : oid(o) {} ;
  OID& ident() { return oid ; }
} ;

typedef HEM<16384, 8, Page, OID> PageIndex ; 

PageIndex pageIndex ;

int main(int argc, char** argv) {

  int number = 0 ;
  Page* page ;

  printf("Taille d'un oid : %d\n", sizeof(OID)) ;
  printf("Taille d'un oidind : %d\n", sizeof(OidInd)) ;
  
  int cpt = atoi(argv[1]) ;
  int tab[cpt] ;
  for (int i = 0; i < cpt; i++) {
    int val = rand() ;
    tab[i] = val ;
    OID oid(val) ;
    if (pageIndex[oid] == NULL) {
      page = new Page(oid) ;
      pageIndex += page ;
      number++ ;
    }
  }

  cout << "Nombre d'objets inseres : " << number << endl ;

  timeBegin() ;
  for (i = 0; i < cpt; i++) {    
    page = pageIndex[OID(tab[i])] ;
  }
  timeEnd() ;
  printf ("Temps ecoule : %f\n", timeDiff()) ;
  
  for (i = 0; i < cpt; i++) {    
    page = pageIndex[OID(tab[i])] ;
    if (page != NULL) {
      if (page->oid.value != tab[i]) {
	cout << "Erreur dans l'index" << endl ;
      }
    } else {
      cout << "Erreur dans l'index" << endl ;
    }
  }


  for (i = 0; i < cpt; i++) {    
    pageIndex -= OID(tab[i]) ;
    for (int j = 0; j < cpt; j++) {    
      page = pageIndex[OID(tab[j])] ;
      if ( (j <= i) && (page != NULL) ) {
	cout << "Erreur dans l'index" << endl ;
      }
      if ( (j > i) && (page == NULL) ) {
	cout << "Erreur dans l'index" << endl ;
      }
    }
  }
  
  for (i = 0; i < cpt; i++) {    
    page = pageIndex[OID(tab[i])] ;
    if (page != NULL) {
      cout << "Erreur dans l'index" << endl ;
    }
  }
}
  

  
